import json
import pytest
import jsonschema

DEFAULT_REQUEST_HEADERS = {
    "Content-type": "application/json",
    "Accept": ["application/json"]
}


def test_router_interfaces(router, client):

    interfaces_list_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "array",
        "items": {
            "type": "object",
            "properties": {
                "name": {"type": "string"},
                "description": {"type": "string"},
                "router": {"type": "string"},
                "bundle": {
                    "type": "array",
                    "items": {"type": "string"}
                },
                "ipv4": {
                    "type": "array",
                    "items": {"type": "string"}
                },
                "ipv6": {
                    "type": "array",
                    "items": {"type": "string"}
                }
            },
            "required": ["name", "description", "ipv4", "router", "ipv6"],
            "additionalProperties": False
        }
    }

    rv = client.post(
        "/data/interfaces/" + router,
        headers=DEFAULT_REQUEST_HEADERS)

    assert rv.status_code == 200
    response = json.loads(rv.data.decode("utf-8"))
    jsonschema.validate(response, interfaces_list_schema)
    assert response  # at least shouldn't be empty


def test_snmp_ids(router, client):

    snmp_id_list_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "array",
        "items": {
            "type": "object",
            "properties": {
                "index": {"type": "integer"},
                "name": {"type": "string"},
                "community": {"type": "string"}
            },
            "required": ["index", "name"],
            "additionalProperties": False
        }
    }

    rv = client.post(
        "/testing/snmp/" + router,
        headers=DEFAULT_REQUEST_HEADERS)

    response = json.loads(rv.data.decode("utf-8"))
    jsonschema.validate(response, snmp_id_list_schema)
    assert response  # at least shouldn't be empty


def test_router_bgp_routes(router, client):

    ROUTERS_WITHOUT_PEERING_DATA = [
        'mx2.bru.be.geant.net'
    ]
    if router in ROUTERS_WITHOUT_PEERING_DATA:
        pytest.skip('%s is not expected to have bgp peers' % router)
        return

    bgp_list_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "definitions": {
            "instance-peering": {
                "type": "object",
                "properties": {
                    "instance": {"type": "string"},
                    "group": {"type": "string"},
                    "description": {"type": "string"},
                    "address": {"type": "string"},
                    "remote-asn": {"type": "integer"},
                    "local-asn": {"type": "integer"}
                },
                # description and-or local-asn is not always present,
                # just based on empirical tests - not a problem
                "required": ["instance", "group", "address", "remote-asn"],
                "additionalProperties": False
            },
            "logical-system-peering": {
                "type": "object",
                "properties": {
                    "logical-system": {"type": "string"},
                    "group": {"type": "string"},
                    "description": {"type": "string"},
                    "address": {"type": "string"},
                    "remote-asn": {"type": "integer"},
                    "local-asn": {"type": "integer"}
                },
                # local/remote-asn and/or description are not always present,
                # just based on empirical tests - not a problem
                "required": ["logical-system", "group", "address"],
                "additionalProperties": False
            },
            "peering": {
                "oneOf": [
                    {"$ref": "#/definitions/instance-peering"},
                    {"$ref": "#/definitions/logical-system-peering"}
                ]
            }
        },
        "type": "array",
        "items": {"$ref": "#/definitions/peering"}
    }

    rv = client.post(
        "/testing/bgp/" + router,
        headers=DEFAULT_REQUEST_HEADERS)

    assert rv.status_code == 200
    response = json.loads(rv.data.decode("utf-8"))
    jsonschema.validate(response, bgp_list_schema)
    assert response  # at least shouldn't be empty
